/*
NITZDemo.c: NITZ adjust the RTC demo
Compiler:
  BC++ 3.1
Compile mode: large

Project:
  NITZDemo.c
  G4500.lib
  GSM_U2.lib
  OS7_COM.lib
  MCU2LIB.lib

Hardware: G-4513 series

[Aug 28, 2014] Modified by William
*/

#include <conio.h>
#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <string.h>
#include "../lib/G4500.h"
#include "../lib/GSM_U2.h"
#include "../lib/OS7_COM.h"
#include "../lib/MCU2LIB.h"

//== this function controls the power of GSM module
void powerFunction(int lv)
{
	if(lv==0)
	{
		MCU2_EnableGSM(0);	//GSM Power OFF
	}else
	{
		MCU2_EnableGSM(1);	//GSM Power ON
	}
}

int main(void)
{
	int iAction=1, quit=1;
	int libVer;
	char libDate[20], model[40];
	int Result=0;
	SYSProfile sysProfile;

	//-- init
	InitLib();

	//-- print G4500 lib. version
	GetLibDate(libDate);
    libVer=GetLibVersion();
	Print("G4500.Lib vesion %X.%02X, Date:%s\r\n", libVer>>8, libVer&0xff, libDate);

	//-- print GSM lib. version
	GM_SYS_GetLibDate(libDate);
	libVer = GM_SYS_GetLibVersion();
	Print("GSM LIB vesion %X.%02X, Date:%s\r\n", libVer>>8, libVer&0xff, libDate);

	//-- print MCU2 lib. version
	MCU2_getLibVer(libDate);
	Print("MCU2 LIB vesion : %s\r\n", libDate);
	
	//-- initial MCU2
	Result = MCU2_init();
	if(Result!=0)
	{
		Print("MCU2 initial fail\r\n");
		return 1;
	}else
	{
		MCU2_getFWVer(libDate);
		Print("MCU2 FW version: %s\r\n", libDate);
	}
	
	//-- Enable Battery Protecting function:
	//		11.1V->close system power, 12.6V->system power recovered
	MCU2_BatteryProtect(1);
	
	//---- init modem
	strcpy(sysProfile.PINCode, "0000");	//The pin code	of SIM card, ex: "0000"
	sysProfile.modemPort = 4;	//modem port number. G-4500 = 4, uP-5000 = 11
	sysProfile.hardware = 0;	//hardware type. 1: G-4500, 2: uPAC-5000, 3: iP-8000, 0: Other
	GM_SYS_SetPowerFunction(powerFunction);  //set power-control function
	if( (Result = GM_SYS_InitModem(sysProfile)) == GM_NOERROR)
		Print("init_modem success!!\r\n");
	else{
		Print("init_modem fail!! return value is %d\r\n", Result);
		return 1;
	}

	//--check Could the modem service?
	while(GM_SYS_CheckModemStatus() != GM_NOERROR)
	{
		Print("wait modem register...\r\n");
		DelayMs(1000);
	}
	Print("modem registered!!\r\n");

	while(iAction!=0)
	{
		iAction=0;
		quit=0;

        Print("\r\n");
		Print("1) NITZ Demo\r\n");
		Print("2) check signal quality\r\n");
		Print("3) check registered?\r\n");
		Print("4) check module name?\r\n");
		Print("5) Close modem\r\n");
        Print("\r\n");
        Print("0) Quits demo program\r\n");
        Print("Choose an option and press [Enter]: ");
        Scanf("%d", &iAction);
        Print("\r\n");

		switch(iAction)
		{
		case 1:
			Print("NITZ Demo\r\n Please wait a minute for RTC updating\r\n");
			GM_SYS_EnableNITZ(1);
			GM_SYS_NITZUpdateRTC();
			while(GM_SYS_CheckNITZ()==2)
			{
				Print("wait NITZ updating RTC\r\n");
				DelayMs(1000);
			}
			if(GM_SYS_CheckNITZ() == 0)
				Print("NITZ update RTC --> fail\r\n");
			else
				Print("NITZ update RTC --> success\r\n");

			break;

		case 2://check signal quality
			Result = GM_SYS_CheckSignal();
			Print("-----------------------------------\r\n");
			Print("signal value = %d\r\n", Result);
			Print("signal quality:\r\n");
			Print("0		-113 dBm or less\r\n");
			Print("1		-111 dBm\r\n");
			Print("2~30		-109... -53 dBm\r\n");
			Print("31		-51 dBm or greater\r\n");
			Print("99		not known or not detectable\r\n");
			Print("-----------------------------------\r\n");
			break;

		case 3://check register value
			Result = GM_SYS_CheckReg();
			Print("-----------------------------------\r\n");
			Print("register value = %d  (0:no register, 1:registered, 2:registering)\r\n", Result);
			Print("register flag:\r\n");
			Print("	0:	not registered\r\n");
			Print("	1:	registered, home network\r\n");
			Print("	2:	not registered, and searching...\r\n");
			Print("	3:	registration denied\r\n");
			Print("	4:	unknown\r\n");
			Print("	5:	registered, roaming\r\n");
			Print("-----------------------------------\r\n");
			break;
		
		case 4:
			GM_SYS_GetModuleName(&model);
			Print("ModuleName=%s\r\n", model);
			break;
		
		case 5:
			GM_SYS_CloseModem(1);
			break;
			
		case 0:
		default:
			quit=1;
			break;

		}//end switch()

		if(!quit)
		{
			Print("Press any key to continue...\r\n");
			Getch();
		}
	}//end while(1)

	//must close before program ending to release you resource
	GM_SYS_CloseModem(0);	//== Close the modem, 0:not turn off modem, 1:turn off modem

	Print("Please press ENTER to exit...\r\n");

	Getch();
	return 0;
}
